{$A+,B-,D+,E+,F-,I+,L+,N-,O-,R-,S-,V-}
{$M 16000,0,100000}


(*
 * SIGDOOR.PAS v1.5A
 * Copyright 1990 by Michael A. Quinlan, all rights reserved.
 * Written in Turbo Pascal v5.5
 * Uses PROKIT v3.2
 * Uses Turbo Professional 5.0 (re-compiled for Turbo Pascal 5.5)
 *
 * Provide access to the diskettes on the PC-SIG CD ROM.
 *
 *)

Program SIGDOOR;

Uses
	Dos,
	TPDos,		{Turbo Professional DOS Unit}
	TPString,	{Turbo Professional String Unit}
	MiniCrt,    {BIOS-only crt functions}
	OpenShare,  {Shared text files}
	MdosIO,     {Dos-level random access files}
	BufIO,      {Buffered record i/o}
	qRead,      {Quick string qReadLn procedure}
	Tools,      {Various utilities}
	ProData,    {ProDoor/pcboard data}
	ProRoot,    {ProKit main support library}
	ProSysf,    {ProKit Status display, function keys, system functions}
	ProUtil,    {ProKit utility library #1}
	ProUtil2,   {proKit utility library #2}
	KitInit;    {ProKit initialization/deinit}

{$DEFINE PCB14}         {Compile for PCBoard 14.0 (otherwise 12.1)}

type textbufpt = ^char;

var dodfn		: string[80];		{ Diskette Description File Filename }
	dixfn		: string[80];		{ Diskette Description Index Filename }
	infomenu	: string[80];		{ PC-SIG Information Menu file name }
	catmenu		: string[80];		{ Diskettes By Category Menu file name }
	zippath		: string[80];		{ Path to PKZIP (or whatever) }
	zipcmd		: string[80];		{ Command for PKZIP (or whatever) }
	libdeffn	: string[80];		{ LIBDEF file name }
	scrpath		: string[80];		{ Path to place archived diskettes into }
	zipext		: string[3];		{ .ZIP file extension }

	textbufs	: word;				{ Buffer size for description file }
	searchpause	: word;				{ Timeout interval for keyword search }

	textbufp	: textbufpt;		{ Ptr to buffer for description file }

	SrchTable	: BTable;			{ Used by Keyword Search }


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  strip_trailing - remove trailing blanks from a string.          *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure strip_trailing(var s : string);
begin
	while (Length(s) > 0) and (s[Length(s)] = ' ') do
		s[0] := Chr(Length(s) - 1);
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  IncPtr - Increment the offset part of a pointer. We know that   *)
(*           we cannot overflow a 64K segment, so we don't check.   *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure IncPtr(var p; inc : word);
var pp : word absolute p;
begin
	pp := pp + inc;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  DecPtr - Decrement the offset part of a pointer.                *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure DecPtr(var p; inc : word);
var pp : word absolute p;
begin
	pp := pp - inc;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  LongToStr - Convert a byte, integer, word, or longint to a      *)
(*              string representation.                              *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function LongToStr(l : longint) : string;
var s : string[11];
begin
	str(l, s);
	LongToStr := s;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  IsNumber - Returns true of the string is a number.              *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function IsNumber(s : string) : boolean;
var i : integer;
begin
	strip_trailing(s);
	IsNumber := FALSE;
	if length(s) <> 0 then begin
		i := 1;
		while s[i] = ' ' do inc(i);
		for i := i to length(s) do
			if (s[i] < '0') or (s[i] > '9') then exit;
		IsNumber := TRUE;
	end;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  lpad - Add blanks to the left of a string until it reaches a    *)
(*         given length.                                            *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function lpad(s : string; n : integer; c : char) : string;
var ss : string;
begin
	ss := s;
	while length(ss) < n do ss := c + ss;
	lpad := ss;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  rpad - Add blanks to the right of a string until it reaches a   *)
(*         given length.                                            *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function rpad(s : string; n : integer; c : char) : string;
var ss : string;
begin
	ss := s;
	while length(ss) < n do ss := ss + c;
	rpad := ss;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  ftime - format the time (from a file) into a date and time      *)
(*          string.                                                 *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function ftime(ldt : longint) : string;
var dt : DateTime;
begin
	UnpackTime(ldt, dt);
	ftime := lpad(LongToStr(dt.Month), 2, '0') + '/'  +
				lpad(LongToStr(dt.Day),   2, '0') + '/'  +
				lpad(LongToStr(dt.Year),  4, '0') + '  ' +
				lpad(LongToStr(dt.Hour),  2, '0') + ':'  +
				lpad(LongToStr(dt.Min),   2, '0');
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  AllocBuf - Call DOS to allocate a buffer outside of the Turbo   *)
(*             Pascal heap. This is used as an I/O buffer, and is   *)
(*             freed as soon as the file is closed.                 *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure AllocBuf(var size : word; var pp);
var p : pointer absolute pp;
var r : registers;
begin
	r.ah := $48;					{ Allocate Memory }
	r.bx := (size + 15) div 16;		{ # of paragraphs to allocate }
	MsDos(r);						{ Have DOS allocate the memeory }
	if (r.flags and $0001) <> 0 then begin	{ request failed }
		if r.bx >= 32 then begin
			r.ah := $48;			{ Allocate Memory }
			size := r.bx * 16;		{ Remember how much we allocated }
			MsDos(r);				{ Allocate all of memory }
			if (r.flags and $0001) <> 0 then begin	{ request failed }
				make_log_entry('Warning: unable to allocate I/O buffer', TRUE);
				r.ax := 0;
				size := 0;
			end;
		end
		else begin
			make_log_entry('Warning: insufficient free memory for I/O buffer', TRUE);
			r.ax := 0;
			size := 0;
		end;
	end;
	p := Addr(Mem[r.ax:0]);
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  FreeBuf - Release a buffer allocated by AllocBuf.               *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure FreeBuf(var pp);
var p : ^char absolute pp;
var r : registers;
begin
	r.ah := $49;					{ Free Allocated Memory }
	r.es := Seg(p^);				{ Segment of block to free }
	MsDos(r);						{ Have DOS free the memeory }
	if (r.flags and 1) <> 0 then 
		make_log_entry('DEBUG: FreeMem failed: ' + LongToStr(r.ax), TRUE);
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  elapsed_time - compute the elapsed time from a given start time.*)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function elapsed_time(start_time : real) : real;
var e : real;
begin
	e := get_time - start_time;
	if e < 0 then elapsed_time := e + (24*60*60)
	else elapsed_time := e;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  cond_enter - ask for an ENTER at the end of the screen.         *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure cond_enter;
begin
	if (user.pagelen <> 0) then begin
		if linenum >= user.pagelen then begin
			force_enter;
			linenum := 0;
		end;
	end;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  load_params - read the SIGDOOR.CNF file to get the various      *)
(*                parameters for the program.                       *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure load_params;
var pf		: TEXT;
	line		: string[80];
	code		: integer;
begin

	{$I-} assign(pf, ParamStr(1)); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + ParamStr(1), TRUE);
		uninit;
		halt(1);
	end;

	FileMode := 0;
	{$I-} reset(pf); {$I+}
	if IOResult <> 0 then begin
		make_log_entry(aMAGENTA+'Cannot open ' + ParamStr(1), TRUE);
		uninit;
		halt(1);
	end;

	ReadLn(pf);						{ Discard 1st line }

	ReadLn(pf, zippath);			{ Line 2 = path to PKZIP }
	strip_trailing(zippath);

	ReadLn(pf, zipcmd);				{ Line 3 = PKZIP parameters }
	strip_trailing(zipcmd);

	ReadLn(pf, dodfn);				{ Line 4 = diskette descriptions file }
	strip_trailing(dodfn);

	ReadLn(pf, line);				{ Line 5 = buffer size }
	Val(line, textbufs, code);
	if code <> 0 then begin
		Make_Log_Entry('Invalid DOD buffer size: ' + line, TRUE);
		uninit;
		Halt(1);
	end;
	if textbufs < 512 then textbufs := 512
	else textbufs := (textbufs div 512) * 512;

	ReadLn(pf, line);					{ Line 6 = search timeout }
	Val(line, searchpause, code);
	if code <> 0 then begin
		make_log_entry('Invalid search pause interval: ' + line, TRUE);
		uninit;
		Halt(1);
	end;

	ReadLn(pf, dixfn);				{ Line 7 = description index file }
	strip_trailing(dixfn);

	ReadLn(pf, infomenu);			{ Line 8 = PC-SIG Information Menu File }
	strip_trailing(infomenu);

	ReadLn(pf, catmenu);			{ Line 9 = Diskettes By Category Menu File }
	strip_trailing(catmenu);

	ReadLn(pf, libdeffn);			{ Line 10 = LIBDEF file name }
	strip_trailing(libdeffn);

	ReadLn(pf, scrpath);			{ Line 11 = path to put archives }
	strip_trailing(scrpath);
	while scrpath[length(scrpath)] = '\' do
		delete(scrpath, length(scrpath), 1);

	if eof(pf) then zipext := 'ZIP'
	else begin
		ReadLn(pf, zipext);			{ Line 12 = ZIP file extension }
		strip_trailing(zipext);
		if zipext = '' then zipext := 'ZIP';
	end;

	close(pf);

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  Menu_Item - set up a menu item for display.                     *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function Menu_Item(n, descr, ncolor, dcolor : string) : string;
begin
	Menu_Item := dcolor + ' (' + ncolor + n + dcolor + ')  ' + descr;
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  search_by_keyword - Search the diskette directory by keyword.   *)
(*                      This version (the 3rd attempt) optimizes by *)
(*                      using the Boyer-Moore string search         *)
(*                      algorithm (from Turbo Professional), AND by *)
(*                      doing high-speed block mode I/O instead of  *)
(*                      slow text I/O.  Doing the block I/O adds a  *)
(*                      number of complications to the code.        *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure search_by_keyword;
var dodf		: file	;		{ Diskette Descriptions }
	keyword		: string;		{ Keyword we are looking for }
	n			: longint;		{ Number of disk blocks we have read }
	h			: longint;		{ Number of "hits" we have so far }
	start_time	: real;			{ Start time of search (or last hit) }
	blksize		: word;			{ Size of blocks for high speed I/O }
	blkoff		: word;			{ Pointer for finding our way within a block }
	blkptr		: textbufpt;	{ Pointer for finding our way within a block }
	charonline  : integer;		{ # chars on this line }
	eoffound	: boolean;		{ TRUE when EOF is found }

	(* ------------------------------------------------------------ *)
	(*                                                              *)
	(*  cleanup - make sure files are closed and the buffer is      *)
	(*            released before exiting search_by_keyword.        *)
	(*                                                              *)
	(* ------------------------------------------------------------ *)
	procedure cleanup;
	begin
		{$I-} close(dodf); {$I+}
		if IOResult <> 0 then ;
		if textbufp <> NIL then begin
			FreeBuf(textbufp);
			textbufp := NIL;
		end;
	end;

begin

	textbufp := NIL;

	{$I-} assign(dodf, dodfn); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + dodfn, TRUE);
		exit;
	end;

	AllocBuf(textbufs, textbufp);
	if textbufp = NIL then begin
		make_log_entry('Cannot allocate buffer for ' + dodfn, TRUE);
		exit;
	end;

	FileMode := 0;
	{$I-} reset(dodf, 1); {$I+}
	if IOResult <> 0 then begin
		Make_log_entry('Cannot open ' + dodfn, TRUE);
		cleanup;
		exit;
	end;

	if length(cmdline) = 0 then
		get_defen('Keyword to search for:');

	if length(cmdline) = 0 then exit;

	get_nextpar;
	keyword := par;

	BMMakeTable(keyword, SrchTable);

	cond_enter;
	displn(aGREEN + 'Searching for "' + keyword +
						'". This may take several minutes...');
	n := 0;
	h := 0;
	start_time := get_time;
	eoffound := FALSE;
	while not eoffound and not Eof(dodf) and not nomore and not dump_user do begin

			{ Read a block of data }

		BlockRead(dodf, textbufp^, textbufs, blksize);

			{ There is, in general, a partial line at the end of the block. }
			{ Find the line and delete it. Reposition the file so we will   }
			{ begin the next block at the start of the truncated line.		}

		eoffound := textbufs <> blksize;
		blkoff := 0;
		blkptr := textbufp;
		incptr(blkptr, blksize-1);		{ Point to last byte of buffer }
		while (blkoff < blksize) and (blkptr^ <> #$0A) do begin
			decptr(blkptr, 1);
			inc(blkoff);
		end;
		Seek(dodf, FilePos(dodf)-blkoff);
		dec(blksize, blkoff);

		n := n + 1;					{ Incr # of blocks read }
		blkptr := textbufp;		{ Point at start of block }
		while (blksize <> 0) and not dump_user do begin		{ Find each match }
			blkoff := BMSearch(blkptr^, blksize, SrchTable, keyword);
			if blkoff <> $FFFF then begin		{ found a match }

					{ We found a match. Find the start of the line containing  }
					{ the match, then display from the start to the end of the }
					{ line. Position to resume the search with the following   }
					{ line.													   }

				incptr(blkptr, blkoff);			{ Point to the match location }
				while (blkoff > 0) and (blkptr^ <> #$0A) do begin
					decptr(blkptr, 1);			{ fine start of the line }
					dec(blkoff);
				end;
				if blkoff <> 0 then begin		{ Skip the #$0A character }
					incptr(blkptr, 1);
					inc(blkoff);
				end;
				disp(aWHITE);			{ Display the line 1 byte at a time }
				charonline := 0;
				while (blkptr^ <> #$0D) do begin
					if charonline < 79 then begin
						inc(charonline);
						disp(blkptr^);
					end;
					incptr(blkptr, 1);
					inc(blkoff);
				end;
				newline;
				if nomore or dump_user then begin
					cleanup;
					exit;
				end;
				dec(blksize, blkoff);		{ Continue search with next line }
				start_time := get_time;		{ Reset timeout clock }
				h := h + 1;					{ Incr number of "hits" }

			end
			else begin

					{ Done searching the block. Check for a timeout. }

				blksize := 0;
				if elapsed_time(start_time) >= searchpause then begin
					get_def(LongToStr(n) + ' of ' +
						LongToStr(FileSize(dodf) div (textbufs - 80) + 1) +
						' blocks processed. Continue searching?', '(Enter)=Yes ');
					get_nextpar;
					if (par[1] = 'N') or dump_user then begin
						cleanup;
						exit;
					end;
					start_time := get_time;
				end;

			end;
		end;
	end;

	if dump_user then exit;

	cond_enter;
	displn(aGREEN + 'Search completed. Found ' + LongToStr(h) + ' matches.');

	cleanup;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  generalized_menu - Display a menu based on a menu file. The     *)
(*                     menu file has a max of 20 entries, each      *)
(*                     entry is two lines long. The first line has  *)
(*                     the description, the 2nd line has the name   *)
(*                     of a file to display when the menu item is   *)
(*                     selected.                                    *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure generalized_menu(menu_title: string; menu_file : string);
const MAX_ITEMS = 20;
var num_items	: 0..MAX_ITEMS;
	cur_item	: 1..MAX_ITEMS;
	sel_item	: integer;
	num_dig		: integer;
	item		: array [1..MAX_ITEMS] of record
					descr : string[70];
					fname : string[80];
				  end;
	tf			: text;
	line		: string[80];
	cl, bg, fg, err : string;
begin

	Assign(tf, menu_file);
	{$I-} Reset(tf); {$I+}
	if ioresult <> 0 then begin
		make_log_entry(aMAGENTA + 'Cannot open ' + menu_file, TRUE);
		exit;
	end;

	num_items := 0;
	while (num_items < MAX_ITEMS) and not eof(tf) do begin
		ReadLn(tf, line);
		strip_trailing(line);
		if line <> '' then begin
			inc(num_items);
			item[num_items].descr := line;
			repeat
				if eof(tf) then begin
					make_log_entry(aMAGENTA + 'Format error in file ' + menu_file, TRUE);
					exit;
				end;
				ReadLn(tf, line);
				strip_trailing(line);
			until line <> '';
			item[num_items].fname := line;
		end;
	end;

	close(tf);

	if num_items < 10 then num_dig := 1
	else if num_items < 100 then num_dig := 2
	else if num_items < 1000 then num_dig := 3
	else if num_items < 10000 then num_dig := 4
	else num_dig := 5;

	bg := aCYAN;
	fg := aYELLOW;
	cl := aYELLOW;
	err := aMAGENTA;

	newline;

	{ main command loop }
	repeat

		{ prompt for input only if there is not a stacked command pending }
		if length(cmdline) = 0 then
		begin
			newline;
			cond_enter;
			displn(fg+menu_title);
			for cur_item := 1 to num_items do begin
				cond_enter;
				displn(Menu_Item(lpad(LongToStr(cur_item),num_dig,'0'), item[cur_item].descr, bg, fg));
			end;
			newline;
			cond_enter;
			displn(Menu_Item('M', 'Return to previous menu', bg, fg));
			cond_enter;
			displn(Menu_Item('Q', 'Quit to PCBoard', bg, fg));
			newline;

			cond_enter;
			repeat
				display_time_left;
				disp(cl+'Command? ');
				get_cmdline;				{get cmdline, map to upper case}
				newline;
			until dump_user or (length(cmdline) > 0);
		end;

		if dump_user then exit;		{leave menu if carrier lost}
		get_nextpar;				{scan next parameter from cmdline into par}

		{ process commands }

		if par = 'M' then exit;
		if par = 'Q' then begin
			uninit;
			halt(0);
		end;

		if not IsNumber(par) then begin
			cond_enter;
			displn(err+'('+par+') is not allowed! Try again:');
		end
		else begin
			sel_item := atoi(par);
			if (sel_item < 1) or (sel_item > num_items) then begin
				cond_enter;
				displn(err+'('+par+') is not allowed! Try again:');
			end
			else begin
				display_file(item[sel_item].fname);
				wait_for_enter;
			end
		end

	until dump_user;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  category_menu - Display diskettes by category. Use the category *)
(*                  menu file.                                      *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure category_menu;
begin
	generalized_menu('List Diskettes By Category:', catmenu);
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  list_description - Display the description for a specific       *)
(*                     diskette. Use the index file to position     *)
(*                     within the description file.                 *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure list_description(disk : string);
var dodf	   	: TEXT;
	dixf	   	: file of Longint;
	line	   	: string;
	diski	   	: word;
	disk4	   	: string[4];
	code		: integer;
	diskpos		: Longint;

	procedure cleanup;
	begin
		{$I-} close(dodf); {$I+}
		if IOResult <> 0 then ;

		{$I-} close(dixf); {$I+}
		if IOResult <> 0 then ;

		if textbufp <> NIL then begin
			FreeBuf(textbufp);
			textbufp := NIL;
		end;
	end;

begin

	textbufp := NIL;

	{$I-} assign(dodf, dodfn); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + dodfn, TRUE);
		exit;
	end;

	if textbufs > 0 then begin
		AllocBuf(textbufs, textbufp);
		if textbufp <> NIL then begin
			settextbuf(dodf, textbufp^, textbufs);
		end;
	end;

	FileMode := 0;
	{$I-} reset(dodf); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + dodfn, TRUE);
		cleanup;
		exit;
	end;

	{$I-} assign(dixf, dixfn); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + dixfn, TRUE);
		exit;
	end;

	FileMode := 0;
	{$I-} reset(dixf); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Cannot open ' + dixfn, TRUE);
		cleanup;
		exit;
	end;

	val(disk, diski, code);

	if (code <> 0) or (diski = 0) or (diski >= FileSize(dixf)) then begin
		cond_enter;
		displn(aMAGENTA+'Invalid disk number: ' + disk);
		cleanup;
		exit;
	end;

	str(diski, disk4);
	if disk4[1] = ' ' then disk4[1] := '0';
	disk4 := lpad(disk4, 4, '0');

	Seek(dixf, diski);
	Read(dixf, diskpos);

	if diskpos = -1 then begin
		cond_enter;
		displn(aMAGENTA+'There is no description for DISK' + disk4);
		cleanup;
		exit;
	end
	else begin
		if not TextSeek(dodf, diskpos) then begin
			make_log_entry('Failure seeking in file ' + dodfn, TRUE);
			cleanup;
			exit;
		end
		else begin
			while not Eof(dodf) and not dump_user do begin
				ReadLn(dodf, line);
				if line <> '' then begin
					if copy(line, 1, 4) = disk4 then begin
						repeat
							if length(line) >= 80 then line[0] := Chr(79);
							if line <> '' then displn(aWHITE + line);
							ReadLn(dodf, line);
						until Eof(dodf) or nomore or dump_user or (copy(line, 1, 4) > disk4);
						cleanup;
						exit;
					end;
				end
			end;
		end;
	end;

	cond_enter;
	displn(aMAGENTA+'Disk ' + disk4 + ' was not found.');
	cleanup;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  list_description_prompt - prompt for a diskette number, then    *)
(*                            call list_description to display the  *)
(*                            description for that diskette.        *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure list_description_prompt;
begin

	if length(cmdline) = 0 then
		get_defen('Disk number:');

	if length(cmdline) = 0 then exit;

	get_nextpar;
	list_description(par);

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  search_dod_menu - Implement the Search Descriptions menu.       *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure search_dod_menu;
var cl, bg, fg, err : string;
begin

	bg := aCYAN;
	fg := aYELLOW;
	cl := aYELLOW;
	err := aMAGENTA;

	newline;

	{ main command loop }
	repeat

		{ prompt for input only if there is not a stacked command pending }
		if length(cmdline) = 0 then
		begin
			newline;
			cond_enter;
			displn(fg+'Search Diskette Descriptions:');
			cond_enter;
			displn(Menu_Item('1', 'Search by Keyword', bg, fg));
			cond_enter;
			displn(Menu_Item('2', 'List diskettes by category', bg, fg));
			cond_enter;
			displn(Menu_Item('3', 'Description of a specific diskette', bg, fg));
			cond_enter;
			displn(Menu_Item('M', 'Return to Main Menu', bg, fg));
			cond_enter;
			displn(Menu_Item('Q', 'Quit to PCBoard', bg, fg));
			newline;

			cond_enter;
			repeat
				display_time_left;
				disp(cl+'Command? ');
				get_cmdline;		{get cmdline, map to upper case}
				newline;
			until dump_user or (length(cmdline) > 0);
		end;

		if dump_user then exit;		{leave menu if carrier lost}
		get_nextpar;				{scan next parameter from cmdline into par}

		{ process commands }
		case par[1] of
			'1': begin
					search_by_keyword;
					wait_for_enter;
				  end;
			'2': begin
					category_menu;
				 	wait_for_enter;
				  end;
			'3': begin
					list_description_prompt;
				 	wait_for_enter;
				  end;

			'M': exit;
			'Q': begin
					uninit;
					halt(0);
				  end;
			else begin
					cond_enter;
					displn(err+'('+par+') is not allowed!  Try again:');
				 end;
		end;

	until dump_user;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  sig_path - Find the path for a diskette, given a diskette       *)
(*             number. Scan the LIBDEF file to find the path.       *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
function sig_path(disk : integer) : string;
var p : string[80];
	n	: integer;
	f	: TEXT;
begin

	{$I-} assign(f, libdeffn); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Unable to open ' + libdeffn, TRUE);
		uninit;
		halt(1);
	end;

	FileMode := 0;
	{$I-} reset(f); {$I+}
	if IOResult <> 0 then begin
		make_log_entry('Unable to open ' + libdeffn, TRUE);
		uninit;
		halt(1);
	end;

	while not eof(f) do begin
		{$I-} ReadLn(f, n, p); {$I+}
		if IOResult = 0 then begin
			if disk <= n then begin
				while (length(p) > 0) and (p[1] = ' ') do delete(p, 1, 1);
				stoupper(p);
				sig_path := p + LongToStr(disk);
				close(f);
				exit;
			end;
		end;
	end;

	close(f);
	sig_path := '';

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  view_diskette - Display the directory of a diskette.            *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure view_diskette(disk : string);
var path : string[80];
	diski	: word;
	disk4	: string[4];
	code	: integer;
	s		: SearchRec;
begin

	val(disk, diski, code);

	if (code <> 0) or (diski = 0) then begin
		cond_enter;
		displn(aMAGENTA+'Invalid disk number: ' + disk);
		exit;
	end;

	str(diski, disk4);
	if disk4[1] = ' ' then disk4[1] := '0';
	disk4 := lpad(disk4, 4, '0');

	path := sig_path(diski);
	if path = '' then begin
		cond_enter;
		displn(aMAGENTA+'Cannot locate DISK' + disk4);
		exit;
	end;

	path := path + '\*.*';
	FindFirst(path, AnyFile, s);
	if DosError <> 0 then begin
		make_log_entry('Cannot locate ' + path, TRUE);
		exit;
	end;

	newline;
	cond_enter;
	displn(aYELLOW + 'Contents of diskette ' + disk4);
	newline;
	while (DosError = 0) and not dump_user and not nomore do begin
		if s.name[1] <> '.' then begin
			if (s.attr and Directory) = Directory then
				displn(aWHITE + '    ' + rpad(s.name,12, ' ') + '   ' + '       <DIR>')
			else
				displn(aWHITE + '    ' + rpad(s.name,12, ' ') + '   ' + 
									lpad(LongToStr(s.size), 12, ' ') + '   ' +
									ftime(s.Time));
		end;
		FindNext(s);
	end;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  view_diskette_prompt - prompt for a diskette number, then call  *)
(*                         view_diskette to display the directory   *)
(*                         for that diskette.                       *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure view_diskette_prompt;
begin

	if length(cmdline) = 0 then
		get_defen('Disk number:');

	if length(cmdline) = 0 then exit;

	get_nextpar;
	view_diskette(par);

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  show_file_info - display file size and estimated download time. *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure show_file_info(file_sz : LongInt);
var	dl_min	: Integer;
begin
	dl_min  := (file_sz div topcps) div 60 + 1;
	displn(aYELLOW + Lpad(LongToStr(file_sz), 6, ' ') + ' bytes, approx. ' +
		Lpad(LongToStr(dl_min), 3, ' ') + ' minutes to download at ' +
		baudrate + 'bps');
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  prepare_diskette - Make an archive of the contents of a diskette*)
(*                     so the user can download it.                 *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure prepare_diskette(disk : string);
var path	: string[80];
	diski	: word;
	disk4	: string[4];
	code	: integer;
	s		: SearchRec;
	zipname	: string[80];
	fullname: string[80];
begin

	val(disk, diski, code);

	if (code <> 0) or (diski = 0) then begin
		cond_enter;
		displn(aMAGENTA+'Invalid disk number: ' + disk);
		exit;
	end;

	zipname  := 'CD-' + LongToStr(diski) + '.' + zipext;
	fullname := scrpath + '\' + zipname;
	if dos_exists(fullname) then begin
		newline;
		cond_enter;
		displn(aYELLOW + 'This diskette is already available as ' + zipname);
		make_log_entry('DISK' + disk4 + ' is already prepared', FALSE);
		cond_enter;
		show_file_info(file_size(fullname));
		exit;
	end;

	str(diski, disk4);
	if disk4[1] = ' ' then disk4[1] := '0';
	disk4 := lpad(disk4, 4, '0');

	path := sig_path(diski);
	if path = '' then begin
		make_log_entry('Cannot locate DISK' + disk4, TRUE);
		exit;
	end;

	path := path + '\*.*';
	FindFirst(path, 0, s);
	if DosError <> 0 then begin
		make_log_entry('Cannot locate ' + path, TRUE);
		exit;
	end;

	cond_enter;
	displn(aGREEN + 'Copying the files. This may take several minutes...');
	SwapVectors;
	Exec(zippath,zipcmd + ' ' + fullname + ' ' + path);
	SwapVectors;

	if DosError <> 0 then begin
		make_log_entry('Archive failed. DosError = ' + LongToStr(DosError), TRUE);
		exit;
	end;

	code := DosExitCode;
	if code <> 0 then begin
		make_log_entry('Archive failed. ExitCode = ' + LongToStr(code), TRUE);
		exit;
	end;

	newline;
	cond_enter;
	displn(aYELLOW + 'Download the diskette as file ' + zipname);
	make_log_entry('DISK' + disk4 + ' Archived into ' + zipname, FALSE);
	cond_enter;
	show_file_info(file_size(fullname));

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  prepare_diskette_prompt - prompt for a diskette number, then    *)
(*                            call prepare_diskette to prepare the  *)
(*                            diskette for downloading.             *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure prepare_diskette_prompt;
begin

	if length(cmdline) = 0 then
		get_defen('Disk number:');

	if length(cmdline) = 0 then exit;

	get_nextpar;
	prepare_diskette(par);

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  display_info_menu - Display PC-SIG information, from the \INFO  *)
(*                      subdirectory on the CD ROM.                 *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure display_info_menu;
begin
	generalized_menu('Display PC-SIG Info:', infomenu);
end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  list_prepared_diskettes - display all CD-nnnn files.            *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure list_prepared_diskettes;
var path : string[80];
	s		: SearchRec;
begin
	path := scrpath + '\CD-*.' + zipext;

	FindFirst(path, 0, s);
	if DosError <> 0 then begin
		make_log_entry('Cannot locate ' + path, TRUE);
		exit;
	end;

	newline;
	cond_enter;
	displn(aGREEN + 'Prepared Diskettes');
	newline;
	while (DosError = 0) and not dump_user and not nomore do begin
		disp(aYELLOW + '  ' + rpad(s.name,12, ' ') + ' ');
		show_file_info(s.size);
		FindNext(s);
	end;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  Main_Menu - Process the main menu.                              *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
procedure Main_Menu;
var cl, bg, fg, err : string;
begin

	bg := aWHITE;
	fg := aCYAN;
	cl := aYELLOW;
	err := aMAGENTA;

	{ main command loop }
	repeat

		{ prompt for input only if there is not a stacked command pending }
		if length(cmdline) = 0 then
		begin
			newline;
			cond_enter;
			displn(fg+'SIGDOOR v1.5A - Access the PC-SIG Disketes on CD ROM');
			cond_enter;
			displn(fg+'Copyright 1990 by Michael A. Quinlan, all rights reserved.');
			newline;
			cond_enter;
			displn(fg+'Main menu:');
			cond_enter;
			displn(Menu_Item('1', 'View information on PC-SIG and this CD ROM', bg, fg));
			cond_enter;
			displn(Menu_Item('2', 'Search the diskette descriptions', bg, fg));
			cond_enter;
			displn(Menu_Item('3', 'View contents of a diskette', bg, fg));
			cond_enter;
			displn(Menu_Item('4', 'Prepare a diskette for downloading', bg, fg));
			cond_enter;
			displn(Menu_Item('5', 'List all prepared diskettes', bg, fg));
			cond_enter;
			displn(Menu_Item('Q', 'Quit to PCBoard', bg, fg));
			newline;

			cond_enter;
			repeat
				display_time_left;
				disp(cl+'Command? ');
				get_cmdline;		{get cmdline, map to upper case}
				newline;
			until dump_user or (length(cmdline) > 0);
		end;

		if dump_user then exit;		{leave menu if carrier lost}
		get_nextpar;				{scan next parameter from cmdline into par}

		{ process commands }
		case par[1] of
			'1': display_info_menu;
			'2': search_dod_menu;
			'3': view_diskette_prompt;
			'4': prepare_diskette_prompt;
			'5': list_prepared_diskettes;

			'Q': exit;
			else begin
					cond_enter;
					displn(err+'('+par+') is not allowed!  Try again:');
				 end;
		end;

	until dump_user;

end;


(* ---------------------------------------------------------------- *)
(*                                                                  *)
(*  Main Program - Initialize, do the main menu, then terminate.    *)
(*                                                                  *)
(* ---------------------------------------------------------------- *)
begin		{main block}
	init;

	load_color_constants('PROCOLOR');
	load_params;

	progname := 'SIGDOOR';

	Main_Menu;

	uninit;
end.
